#ifndef _AUDIOSOURCEFLV_H_
#define _AUDIOSOURCEFLV_H_


// AAC format tags from wmcodecdsp.h
#define WAVE_FORMAT_RAW_AAC1       0x00FF
#define WAVE_FORMAT_MPEG_ADTS_AAC  0x1600


class AudioTrackFLV
{
private:
	DataVector*     dv;
	
	unsigned char*  pHeader;
	int             iHeaderLen;

	bool			ParseASC();
	bool            FormatADPCM();
	bool            FormatMP3();
	bool            FormatAAC();

public:
	AudioTrackFLV();
	~AudioTrackFLV();

	unsigned char*  pFormat;
	int             iFormatLen;

	FLVSampleInfo*  sample_list;
	DWORD           dwLength;
	DWORD           dwMaxSample;
	DWORD           dwTotalBytes;
	double          Duration;

	DWORD           dwRate;
	DWORD           dwScale;

	unsigned char   bFormat;
	unsigned char   bFormatHi;
	bool            bHasVSB;
	bool            bDeliverRawAAC;

	bool            ParseTag(WinFile &mFile, const FLVTag &tag);
	bool            Finish();
};


class AudioSourceFLV : public vdxunknown<IVDXStreamSource>, public IVDXStreamSourceV3, public IVDXAudioSource {

public:
	AudioSourceFLV(InputFileFLV *const pp, AudioTrackFLV *const audio);
	~AudioSourceFLV();

	// IVDXUnknown
	int         VDXAPIENTRY AddRef();
	int         VDXAPIENTRY Release();
	void*       VDXAPIENTRY AsInterface(uint32 iid);

	// IVDXStreamSource
	void        VDXAPIENTRY GetStreamSourceInfo(VDXStreamSourceInfo&);
	bool        VDXAPIENTRY Read(sint64 lStart, uint32 lCount, void *lpBuffer, uint32 cbBuffer, uint32 *lBytesRead, uint32 *lSamplesRead);
	const void* VDXAPIENTRY GetDirectFormat();
	int         VDXAPIENTRY GetDirectFormatLen();
	ErrorMode   VDXAPIENTRY GetDecodeErrorMode();
	void        VDXAPIENTRY SetDecodeErrorMode(ErrorMode mode);
	bool        VDXAPIENTRY IsDecodeErrorModeSupported(ErrorMode mode);
	bool        VDXAPIENTRY IsVBR();
	sint64      VDXAPIENTRY TimeToPositionVBR(sint64 us);
	sint64      VDXAPIENTRY PositionToTimeVBR(sint64 samples);

	// IVDXStreamSourceV3
	void        VDXAPIENTRY GetStreamSourceInfoV3(VDXStreamSourceInfoV3&);

	// IVDXAudioSource
	void        VDXAPIENTRY GetAudioSourceInfo(VDXAudioSourceInfo& info);


private:
	InputFileFLV*   const parentPtr;
	AudioTrackFLV*  const audio;

	IVDXStreamSource::ErrorMode errmode;

	unsigned char*  adpcm_buf;
	unsigned char*  adpcm_out;
	unsigned long   adpcm_ptr;
	unsigned long   adpcm_len;

	int     adpcm_bits(int nBits);
	void    adpcm_decode();
};

#endif	// _AUDIOSOURCEFLV_H_

