///////////////////////////////////////////////////////////////////////
// FLV plugin for VirtualDub 1.8.8+
// Copyright (C) 2010-2012 fccHandler
// Copyright (C) 1998-2012 Avery Lee
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////

// Generic DataVector

#include <string.h>
#include "DataVector.h"


DataVector::DataVector(int item_size)
	: _item_size    (item_size)
	, first         (NULL)
	, last          (NULL)
	, count         (0)
{
}


DataVector::~DataVector()
{
	DataVectorBlock *i, *j;

	i = last;
	while (j = i)
	{
		i = j->prev;
		delete j;
	}
}


void DataVector::Add(const void *pp)
{
	if (last == NULL)
	{
		DataVectorBlock *ib = new DataVectorBlock;

		memcpy(ib->heap, pp, _item_size);
		ib->next = NULL;
		ib->size = _item_size;

		if (first == NULL)
		{
			// New "first" block
			first = ib;
			first->prev = NULL;
		}
		else
		{
			// New "last" block
			last = ib;
			first->next = last;
			last->prev = first;
		}
	}
	else
	{
		const int remains = DataVectorBlock::BLOCK_SIZE - last->size;
		int len = _item_size;

		if (len > remains)
		{
			// New "last" block
			DataVectorBlock *ib = new DataVectorBlock;

			last->next = ib;
			ib->prev = last;
			ib->next = NULL;
			ib->size = 0;

			if (remains > 0)
			{
				memcpy(last->heap + last->size, pp, remains);
				pp = (char *)pp + remains;
				last->size += remains;
				len -= remains;
			}

			last = ib;
		}

		memcpy(last->heap + last->size, pp, len);
		last->size += len;
	}

	++count;
}


void *DataVector::MakeArray() const
{
	if (count > 0)
	{
		char *arr = new char[count * _item_size];
		char *ptr = arr;

		const DataVectorBlock *dvb = first;

		while (dvb)
		{
			memcpy(ptr, dvb->heap, dvb->size);
			ptr += dvb->size;

			dvb = dvb->next;
		}

		return arr;
	}
	return NULL;
}

