///////////////////////////////////////////////////////////////////////
// FLV plugin for VirtualDub 1.8.8+
// Copyright (C) 2010-2012 fccHandler
// Copyright (C) 1998-2012 Avery Lee
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////

// File Information dialog (part of InputFileFLV class)

#if defined(_MSC_VER) && (_MSC_VER >= 1400)
	#define _CRT_SECURE_NO_WARNINGS
#endif

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <commctrl.h>
#include <vfw.h>
#include <stdio.h>

#include "vd2\plugin\vdinputdriver.h"
#include "Unknown.h"

#include "DataVector.h"
#include "WinFile.h"
#include "FLVTypes.h"
#include "InputFileFLV.h"
#include "AudioSourceFLV.h"
#include "VideoSourceFLV.h"
#include "InputOptionsFLV.h"
#include "Resource.h"


char *InputFileFLV::time_to_string(double seconds)
{
	// There are 86400 seconds in a day,
	// about 32 million seconds in a year.
	// Since we are unlikely to encounter videos
	// of such an extreme duration, we can easily
	// get away with using longs for this math...

	const int buf_size = 32;
	char *ret = new char[buf_size];

	if (ret != NULL)
	{
		int hours, mins, mils;
		int secs = (int)seconds;

		hours = secs / 3600;
		secs  = secs % 3600;
		mins  = secs / 60;
		secs  = secs % 60;
		mils  = (int)(seconds * 1000.0 + 0.5);
		mils  = mils % 1000;

		if (hours > 0)
		{
			_snprintf(ret, buf_size, "%i:%02i:%02i.%03i", hours, mins, secs, mils);
		}
		else
		{
			_snprintf(ret, buf_size, "%i:%02i.%03i", mins, secs, mils);
		}

		ret[buf_size - 1] = '\0';
	}

	return ret;
}


void InputFileFLV::UpdateDlgAudio(HWND hDlg, const AudioTrackFLV *at)
{
	const WAVEFORMATEX *wfex;
	char buf[128];
	ACMFORMATTAGDETAILSA aftd = { 0 };
	
	if (at == NULL) return;

	wfex = (WAVEFORMATEX *)at->pFormat;

	unsigned int i;
	char *s;

/**************************************************************************
 *      Sampling rate, channels:
 **************************************************************************/

	i = _snprintf(buf, sizeof(buf), "%lu Hz, ", wfex->nSamplesPerSec);
	if (i < sizeof(buf))
	{
		if (wfex->nChannels == 1) {
			_snprintf(buf + i, sizeof(buf) - i, "Mono");
		} else if (wfex->nChannels == 2) {
			_snprintf(buf + i, sizeof(buf) - i, "Stereo");
		} else {
			_snprintf(buf + i, sizeof(buf) - i, "%hu channels", wfex->nChannels);
		}
	}
	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_AUDIO_SAMPLINGRATE, buf);

/**************************************************************************
 *      Sample precision:
 **************************************************************************/

	if (wfex->wFormatTag == WAVE_FORMAT_PCM && wfex->wBitsPerSample) {
		_snprintf(buf, sizeof(buf), "%hu-bit", wfex->wBitsPerSample);
	} else {
		_snprintf(buf, sizeof(buf), "N/A");
	}
	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_AUDIO_PRECISION, buf);

/**************************************************************************
 *      ACM codec (tag):
 **************************************************************************/

	aftd.cbStruct    = sizeof(aftd);
	aftd.dwFormatTag = wfex->wFormatTag;

	if (acmFormatTagDetailsA(NULL, &aftd,
		ACM_FORMATTAGDETAILSF_FORMATTAG) == MMSYSERR_NOERROR)
	{
		_snprintf(buf, sizeof(buf), "%s (0x%04X)", aftd.szFormatTag, wfex->wFormatTag);
	} else {
		_snprintf(buf, sizeof(buf), "Missing codec (0x%04X)", wfex->wFormatTag);
	}
	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_AUDIO_COMPRESSION, buf);

/**************************************************************************
 *      Block layout:
 **************************************************************************/

	_snprintf(buf, sizeof(buf), "%lu blocks", at->dwLength);

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_AUDIO_LAYOUT, buf);

/**************************************************************************
 *      Length (duration):
 **************************************************************************/

	s = time_to_string(at->Duration);
	if (s != NULL)
	{
		_snprintf(buf, sizeof(buf), "%lu samples (%s)",
			(unsigned long)(at->Duration * wfex->nSamplesPerSec + 0.5), s);
		delete[] s;
	}
	else
	{
		_snprintf(buf, sizeof(buf), "%lu samples (unknown)",
			(unsigned long)(at->Duration * wfex->nSamplesPerSec + 0.5));
	}

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_AUDIO_LENGTH, buf);

/**************************************************************************
 *      Min/avg/max(total) block size:
 **************************************************************************/

	if (wfex->wFormatTag == WAVE_FORMAT_PCM)
	{
		_snprintf(buf, sizeof(buf), "%hu / %hu / %hu (%lu)",
			wfex->nBlockAlign, wfex->nBlockAlign, wfex->nBlockAlign,
			at->dwTotalBytes
		);
	}
	else
	{
		unsigned long dwMinBlock;
		unsigned long dwAvgBlock;
		unsigned long dwMaxBlock;

		if (at->dwLength != 0)
		{
			dwMinBlock = at->sample_list[0].size;
			dwAvgBlock = dwMinBlock;
			dwMaxBlock = dwMinBlock;

			for (i = 1; i < at->dwLength; ++i)
			{
				unsigned long dwSize = at->sample_list[i].size;
				if (dwSize < dwMinBlock) dwMinBlock = dwSize;
				if (dwSize > dwMaxBlock) dwMaxBlock = dwSize;
			}

			if (wfex->wFormatTag == WAVE_FORMAT_RAW_AAC1 && !at->bDeliverRawAAC)
			{
				dwMinBlock += 7;
				dwMaxBlock += 7;
			}

			dwAvgBlock = (unsigned long)
				((double)at->dwTotalBytes / (double)at->dwLength + 0.5);
		}
		else
		{
			dwMinBlock = 0;
			dwAvgBlock = 0;
			dwMaxBlock = 0;
		}

		_snprintf(buf, sizeof(buf), "%lu / %lu / %lu (%lu)",
			dwMinBlock, dwAvgBlock, dwMaxBlock,
			at->dwTotalBytes
		);
	}

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_AUDIO_FRAMESIZES, buf);

/**************************************************************************
 *      Data rate:
 **************************************************************************/

	if (at->Duration)
		i = (unsigned int)((8.0 * at->dwTotalBytes) / at->Duration + 0.5);
	else
		i = 0;

	_snprintf(buf, sizeof(buf), "%u bits/sec", i);

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_AUDIO_DATARATE, buf);
}


void InputFileFLV::UpdateDlgVideo(HWND hDlg, const VideoTrackFLV *vt)
{
	const BITMAPINFOHEADER *bmih;
	char buf[128];
	char *s;
	double fps;
	unsigned int i;
	
	__int64       i64VideoBytes       = 0;
	unsigned long ulFrameCount[2]     = { 0, 0 };
	unsigned long ulFrameMinSize[2]   = { 0xFFFFFFFF, 0xFFFFFFFF };
	unsigned long ulFrameMaxSize[2]   = { 0, 0 };
	unsigned long ulFrameTotalSize[2] = { 0, 0 };

	if (vt == NULL) return;

	bmih = &vt->bmih;

/**************************************************************************
 *      Frame size, average fps:
 **************************************************************************/

	if (vt->dwScale)
		fps = (double)vt->dwRate / (double)vt->dwScale;
	else
		fps = 0;

	_snprintf(buf, sizeof(buf), "%ldx%ld, %.3f fps",
		bmih->biWidth, bmih->biHeight, fps);

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_VIDEO_FORMAT, buf);

/**************************************************************************
 *      Number of frames (duration):
 **************************************************************************/

	s = time_to_string((double)vt->dwPerceivedLength / fps);
	if (s != NULL)
	{
		_snprintf(buf, sizeof(buf), "%lu (%s)", vt->dwPerceivedLength, s);
		delete[] s;
	}
	else
	{
		_snprintf(buf, sizeof(buf), "%lu (unknown)", vt->dwPerceivedLength);
	}

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_VIDEO_NUMFRAMES, buf);

/**************************************************************************
 *      VCM codec (fourcc):
 **************************************************************************/

	if (vt->hCodec != NULL)
	{
		ICINFO ici;

		ici.dwSize = sizeof(ici);
		if (ICGetInfo(vt->hCodec, &ici, sizeof(ici))
			>= FIELD_OFFSET(ICINFO, szDriver))
		{
			i = _snprintf(buf, sizeof(buf), "%ls ", ici.szDescription);
		}
		else
		{
			i = _snprintf(buf, sizeof(buf), "Unknown ");
		}
	}
	else
	{
		i = _snprintf(buf, sizeof(buf), "Missing codec ");
	}

	if (i < sizeof(buf))
	{
		unsigned char fcc[5];
		int n;

		*(DWORD *)fcc = bmih->biCompression;
		for (n = 0; n < 4; ++n)
		{
			if (fcc[n] < 32) fcc[n] = '?';
		}
		fcc[4] = '\0';
		_snprintf(buf + i, sizeof(buf) - i, "(%s)", (char *)fcc);
	}

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_VIDEO_COMPRESSION, buf);

/**************************************************************************
 *      Number of key frames:
 **************************************************************************/

	if (vt->dwActualLength == 0)
	{
		ulFrameMinSize[0] = 0;
		ulFrameMinSize[1] = 0;
	}

	for (i = 0; i < vt->dwActualLength; ++i)
	{
		const FLVSampleInfo& fsi = vt->sample_list[i];
		int iFrameType = (fsi.size & FLV_KEYFRAME)? 0: 1;
		unsigned long ulSize = fsi.size & (~FLV_KEYFRAME);

		i64VideoBytes += ulSize;

		++ulFrameCount[iFrameType];
		ulFrameTotalSize[iFrameType] += ulSize;

		if (ulSize < ulFrameMinSize[iFrameType])
			ulFrameMinSize[iFrameType] = ulSize;

		if (ulSize > ulFrameMaxSize[iFrameType])
			ulFrameMaxSize[iFrameType] = ulSize;
	}

	_snprintf(buf, sizeof(buf), "%lu", ulFrameCount[0]);

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_VIDEO_NUMKEYFRAMES, buf);

/**************************************************************************
 *      Min/avg/max(total) key frame size:
 **************************************************************************/

	if (ulFrameCount[0] > 0)
	{
		_snprintf(buf, sizeof(buf), "%lu / %lu / %lu (%lu)",
			ulFrameMinSize[0],
			(unsigned long)((double)ulFrameTotalSize[0] / (double)ulFrameCount[0] + 0.5),
			ulFrameMaxSize[0],
			ulFrameTotalSize[0]
		);
	} else {
		_snprintf(buf, sizeof(buf), "(no key frames)");
	}

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_VIDEO_KEYFRAMESIZES, buf);

/**************************************************************************
 *      Min/avg/max(total) delta frame size:
 **************************************************************************/

	if (ulFrameCount[1] > 0)
	{
		_snprintf(buf, sizeof(buf), "%lu / %lu / %lu (%lu)",
			ulFrameMinSize[1],
			(unsigned long)((double)ulFrameTotalSize[1] / (double)ulFrameCount[1] + 0.5),
			ulFrameMaxSize[1],
			ulFrameTotalSize[1]
		);
	} else {
		_snprintf(buf, sizeof(buf), "(no delta frames)");
	}

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_VIDEO_NONKEYFRAMESIZES, buf);

/**************************************************************************
 *      Data rate:
 **************************************************************************/

	// bits * (frames per sec) / frames = bits per sec
	// (a / b) / c  =  a / (b * c)
	// a / (b / c)  =  (a * c) / b

	i = 0;
	if (fps)
	{
		// calculate the duration in seconds:
		fps = (double)vt->dwActualLength / fps;
		if (fps)
			i = (unsigned int)((8.0 * i64VideoBytes) / fps + 0.5);
	}

	_snprintf(buf, sizeof(buf), "%u bits/sec", i);

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_VIDEO_DATARATE, buf);
}


INT_PTR APIENTRY InputFileFLV::InfoDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {

    case WM_INITDIALOG:
    {
		InputFileFLV *thisPtr = (InputFileFLV *)lParam;
		InputOptionsFLV::InitDialogTitle(hDlg, " - File Information");

		UpdateDlgVideo(hDlg, thisPtr->video);
		UpdateDlgAudio(hDlg, thisPtr->audio);

		return TRUE;
	}

	case WM_COMMAND:
		// Note: "IDCANCEL" is the [X] button
		// Return TRUE if we process the message

		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)
		{
			EndDialog(hDlg, TRUE);
			return TRUE;
		}
		break;
    }

    return FALSE;
}


