///////////////////////////////////////////////////////////////////////
// FLV plugin for VirtualDub 1.8.8+
// Copyright (C) 2010-2012 fccHandler
// Copyright (C) 1998-2012 Avery Lee
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////

// InputFileFLV implementation

#if defined(_MSC_VER) && (_MSC_VER >= 1400)
	#define _CRT_SECURE_NO_WARNINGS
#endif

#include <windows.h>
#include <commctrl.h>
#include <vfw.h>
#include <stdio.h>
#include <crtdbg.h>

#include "vd2\plugin\vdinputdriver.h"
#include "Unknown.h"

#include "WinFile.h"
#include "DataVector.h"

#include "FLVTypes.h"
#include "InputFileFLV.h"
#include "InputOptionsFLV.h"
#include "VideoSourceFLV.h"
#include "AudioSourceFLV.h"
#include "Resource.h"


HMODULE g_hModule;

BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	if (fdwReason == DLL_PROCESS_ATTACH)
	{
		g_hModule = (HMODULE)hinstDLL;
	}

	return TRUE;
}


///////////////////////////////////////////////////////////////////////
//
//						InputFileFLV
//
///////////////////////////////////////////////////////////////////////

InputFileFLV::InputFileFLV(const VDXInputDriverContext& context, uint32 flags)
	: mContext  (context)
	, uiFlags   (flags)
	, video     (NULL)
	, audio     (NULL)
{
	memset(&flv_header, 0, sizeof(flv_header));

_RPT0(_CRT_WARN, "InputFileFLV::constructed\n");
}

InputFileFLV::~InputFileFLV()
{
_RPT0(_CRT_WARN, "InputFileFLV::destructed\n");

	delete audio;
	delete video;
}


int InputFileFLV::AddRef()
{
_RPT0(_CRT_WARN, "InputFileFLV::AddRef\n");
	return vdxunknown<IVDXInputFile>::AddRef();
}


int InputFileFLV::Release()
{
_RPT0(_CRT_WARN, "InputFileFLV::Release\n");
	return vdxunknown<IVDXInputFile>::Release();
}


void *VDXAPIENTRY InputFileFLV::AsInterface(uint32 iid)
{
	return vdxunknown<IVDXInputFile>::AsInterface(iid);
}


long InputFileFLV::ReadData(void *lpBuffer, __int64 offset, int bytes)
{
	mFile.Seek(offset);
	return mFile.Read(lpBuffer, bytes);
}


INT_PTR CALLBACK InputFileFLV::ParseDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	InputFileFLV *thisPtr = (InputFileFLV *)GetWindowLongPtrA(hDlg, DWLP_USER);

	switch(uMsg) {

	case WM_INITDIALOG:
	{
		SetWindowLongPtrA(hDlg, DWLP_USER, lParam);
		thisPtr = (InputFileFLV *)lParam;

		InputOptionsFLV::InitDialogTitle(hDlg, " by fccHandler");

		SendMessageA(GetDlgItem(hDlg, IDC_PROGRESS1), PBM_SETRANGE, 0, MAKELPARAM(0, 16384));
		SetTimer(hDlg, 1, 250, (TIMERPROC)NULL);
		ShowWindow(hDlg, SW_SHOW);

		return TRUE;
	}

	case WM_TIMER:
	{
		char buf[64];
		__int64 file_cpos = thisPtr->mFile.Pos();
		__int64 file_size = thisPtr->mFile.Size();

		SendMessageA(GetDlgItem(hDlg, IDC_PROGRESS1), PBM_SETPOS,
			(WPARAM)((file_cpos * 16384I64) / file_size), 0);

		sprintf(buf, "%I64iK of %I64iK",
			file_cpos / 1024, (file_size + 512) / 1024
		);

		SendDlgItemMessageA(hDlg, IDC_CURRENT_VALUE, WM_SETTEXT, 0, (LPARAM)buf);
		return TRUE;
	}

	case WM_COMMAND:
		if (LOWORD(wParam) == IDCANCEL) thisPtr->fAbort = true;
		return TRUE;
	}

	return FALSE;
}


bool InputFileFLV::guiDlgMessageLoop(HWND hDlg)
{
	MSG msg;

	while(PeekMessageA(&msg, NULL, 0, 0, PM_REMOVE))
	{
		if (msg.message == WM_QUIT) return false;

		if (!hDlg || !IsWindow(hDlg) || !IsDialogMessageA(hDlg, &msg))
		{
			TranslateMessage(&msg);
			DispatchMessageA(&msg);
		}
	}

	return true;
}


BOOL CALLBACK InputFileFLV::FindContextWindow(HWND hwnd, LPARAM lParam)
{
	if (GetParent(hwnd) == NULL)
	{
		*(HWND *)lParam = hwnd;
		return FALSE;
	}
	return TRUE;
}


bool InputFileFLV::ParseFLVTags()
{
	HWND hwndParent;
	HWND hwndProgress;
	FLVTag tag;

	__int64 blockstart;
	unsigned int blocksize;
	
	hwndParent   = NULL;
	hwndProgress = NULL;
	fAbort       = false;

	if (!(uiFlags & IVDXInputFileDriver::kFlagQuiet))
	{
		// Get a handle to the main VirtualDub window
		EnumThreadWindows(GetCurrentThreadId(), FindContextWindow, (LPARAM)&hwndParent);
		if (hwndParent == NULL) hwndParent = GetActiveWindow();

		if (hwndParent != NULL)
		{
			EnableWindow(hwndParent, FALSE);

			// Pop up parse dialog
			hwndProgress = CreateDialogParamA(g_hModule,
				MAKEINTRESOURCEA(IDD_PROGRESS), hwndParent,
				ParseDialogProc, (LPARAM)this);
		}
	}

	while (!mFile.EndOfFile())
	{
		// Update the progress dialog
		if (hwndProgress != NULL)
		{
			guiDlgMessageLoop(hwndProgress);
			if (fAbort) break;
		}

		if (mFile.Read(&tag, sizeof(tag)) < sizeof(tag)) break;

		blockstart = mFile.Pos();
		blocksize  = tag.DataSize[0];
		blocksize  = (blocksize << 8) + tag.DataSize[1];
		blocksize  = (blocksize << 8) + tag.DataSize[2];

		if (tag.TagType == FLV_VIDEODATA)
		{
			if (video == NULL)
			{
				video = new VideoTrackFLV;
				if (video == NULL)
				{
					mContext.mpCallbacks->SetErrorOutOfMemory();
					break;
				}
			}
			if (!video->ParseTag(mFile, tag)) break;
		}
		else if (tag.TagType == FLV_AUDIODATA)
		{
			if (audio == NULL)
			{
				audio = new AudioTrackFLV;
				if (audio == NULL)
				{
					mContext.mpCallbacks->SetErrorOutOfMemory();
					break;
				}
			}
			audio->ParseTag(mFile, tag);
		}

		mFile.Seek(blockstart + blocksize);
	}

	if (hwndProgress != NULL)
	{
		EnableWindow(hwndParent, TRUE);
		DestroyWindow(hwndProgress);
	}

	return (video != NULL);
}


void InputFileFLV::ParseFileRoot()
{
	bool parse_ok = false;

	mFile.Seek(0);

	if (mFile.Read(&flv_header, sizeof(flv_header)) >= sizeof(flv_header))
	{
		// Ignore the TypeFlags; they are sometimes wrong.
		if (flv_header.Signature[0] == 'F' &&
		    flv_header.Signature[1] == 'L' &&
		    flv_header.Signature[2] == 'V' &&
		    flv_header.Version == 1 )
		{
			unsigned long pdata =
				((flv_header.DataOffset >> 24) & 0x000000FF) |
				((flv_header.DataOffset >>  8) & 0x0000FF00) |
				((flv_header.DataOffset <<  8) & 0x00FF0000) |
				((flv_header.DataOffset << 24) & 0xFF000000) ;

			if (pdata >= sizeof(flv_header))
			{
				mFile.Seek(pdata);
				parse_ok = ParseFLVTags();
			}
		}
	}

	if (!parse_ok)
	{
		mContext.mpCallbacks->SetError("Could not parse FLV file.");
		return;
	}

	// Voila, the FLV is parsed!

	if (video != NULL)
	{
		if (!video->Finish())
		{
			mContext.mpCallbacks->SetError("Unsupported FLV file.");
		}
	}
}


void InputFileFLV::Init(const wchar_t *szFile, IVDXInputOptions *opts)
{
	bool fDeliverRawAAC;

	if (opts != NULL)
	{
		// Get options from VirtualDub
		const InputOptionsFLV *pOptions = (InputOptionsFLV *)opts;
		fDeliverRawAAC = pOptions->opts.fDeliverRawAAC;
	}
	else
	{
		// Get options (if any) from the registry
		InputOptionsFLV pOptions;
		fDeliverRawAAC = pOptions.opts.fDeliverRawAAC;
	}

	if (!mFile.OpenW(szFile))
	{
		mContext.mpCallbacks->SetError("Cannot open file: %ls", szFile);
		return;
	}

	ParseFileRoot();

	if (audio != NULL)
	{
		if (audio->bFormatHi == FLV_AUDIO_AAC)
		{
			audio->bDeliverRawAAC = fDeliverRawAAC;
		}

		if (!audio->Finish())
		{
			delete audio;
			audio = NULL;
		}
	}
}


bool InputFileFLV::Append(const wchar_t *szFile)
{
	return false;
}


bool InputFileFLV::PromptForOptions(VDXHWND hwndParent, IVDXInputOptions **ppOptions)
{
	InputOptionsFLV *p = new InputOptionsFLV;

	*ppOptions = p;
	if (p == NULL) return false;

	p->AddRef();

	p->OptionsDlg((HWND)hwndParent);
	return true;
}


bool InputFileFLV::CreateOptions(const void *buf, uint32 len, IVDXInputOptions **ppOptions)
{
	InputOptionsFLV *p = new InputOptionsFLV;

	*ppOptions = p;
	if (p == NULL) return false;

	p->AddRef();

	if (len > p->opts.len) len = p->opts.len;
	memcpy(&(p->opts), buf, len);

	return true;
}


void InputFileFLV::DisplayInfo(VDXHWND hwndParent)
{
    DialogBoxParam(g_hModule, MAKEINTRESOURCE(IDD_FLV_INFO),
		(HWND)hwndParent, InfoDlgProc, (LPARAM)this);
}


bool InputFileFLV::GetVideoSource(int index, IVDXVideoSource **ppVS)
{
	*ppVS = NULL;
	if (index) return false;

	if (video != NULL)
	{
		*ppVS = new VideoSourceFLV(this, video);
		if (*ppVS)
		{
			(*ppVS)->AddRef();
			return true;
		}
	}
	return false;
}


bool InputFileFLV::GetAudioSource(int index, IVDXAudioSource **ppAS)
{
	*ppAS = NULL;
	if (index) return false;

	if (audio != NULL)
	{
		*ppAS = new AudioSourceFLV(this, audio);
		if (*ppAS)
		{
			(*ppAS)->AddRef();
			return true;
		}
	}
	return false;
}


///////////////////////////////////////////////////////////////////////
//
//						InputFileDriverFLV
//
///////////////////////////////////////////////////////////////////////


class InputFileDriverFLV : public vdxunknown<IVDXInputFileDriver>
{

public:
	InputFileDriverFLV(const VDXInputDriverContext& context);

	int		VDXAPIENTRY DetectBySignature(const void *pHeader, sint32 nHeaderSize, const void *pFooter, sint32 nFooterSize, sint64 nFileSize);
	bool	VDXAPIENTRY CreateInputFile(uint32 flags, IVDXInputFile **ppFile);

protected:
	const VDXInputDriverContext& mContext;
};


InputFileDriverFLV::InputFileDriverFLV(const VDXInputDriverContext& context)
	: mContext(context)
{
}


int VDXAPIENTRY InputFileDriverFLV::DetectBySignature(const void *pHeader, sint32 nHeaderSize, const void *pFooter, sint32 nFooterSize, sint64 nFileSize)
{
	return -1;
}


bool VDXAPIENTRY InputFileDriverFLV::CreateInputFile(uint32 flags, IVDXInputFile **ppFile)
{
	*ppFile = new InputFileFLV(mContext, flags);
	if (*ppFile == NULL) return false;

	(*ppFile)->AddRef();
	return true;
}


///////////////////////////////////////////////////////////////////////

bool VDXAPIENTRY flv_create(const VDXInputDriverContext *pContext, IVDXInputFileDriver **ppDriver)
{
	*ppDriver = new InputFileDriverFLV(*pContext);
	if (*ppDriver == NULL) return false;

	(*ppDriver)->AddRef();
	return true;
}


// Signature 'FLV'
// Version   1
// 
// flags: 00000.0.    reserved (0)
//        .....x..    audio tags are present
//        .......x    video tags are present
//
// data offset   00 00 00 09
//
// (ignore the flags; they are sometimes wrong)

const uint8 flv_sig[] = {
	'F', 0xFF, 'L', 0xFF, 'V', 0xFF,	// Signature = 'FLV'
	1, 0xFF,							// Version = 1
	0, 0x00,							// type flags
	0, 0x80								// data offset not negative
};


const VDXInputDriverDefinition FLV_input = {
	sizeof(VDXInputDriverDefinition),
	VDXInputDriverDefinition::kFlagSupportsVideo |
		VDXInputDriverDefinition::kFlagSupportsAudio,
	0,
	sizeof flv_sig,
	flv_sig,
	L"*.flv",
	L"Flash Video files (*.flv)|*.flv",
	L"flv",
	flv_create
};


// (major<<24) + (minor<<16) + build.  1.4.1000 would be 0x010403E8.

//	xxxxxxxx ........ ........ ........		major
//	........ xxxxxxxx ........ ........		minor
//	........ ........ xxxxxxxx xxxxxxxx		build

const VDXPluginInfo FLV_info = {
	sizeof(VDXPluginInfo),              // size
	L"FLV plugin",                      // name
	L"fccHandler",                      // author
	L"Loads Flash Video files.",        // description
	0x02040000,                         // (major<<24) + (minor<<16) + build
	kVDXPluginType_Input,               // type
	1,                                  // flags
	10,									// version required
	kVDXPlugin_APIVersion,              // version used
	2,									// version required
	kVDXPlugin_InputDriverAPIVersion,   // version used
	&FLV_input
};


///////////////////////////////////////////////////////////////////////

const VDXPluginInfo *const kPlugins[] = {
	&FLV_info,
	NULL
};


extern "C" const VDXPluginInfo *const * VDXAPIENTRY VDGetPluginInfo() {
	return kPlugins;
}
