///////////////////////////////////////////////////////////////////////
// FLV plugin for VirtualDub 1.8.8+
// Copyright (C) 2010-2012 fccHandler
// Copyright (C) 1998-2012 Avery Lee
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////

// IVDXInputOptions implementation

#if defined(_MSC_VER) && (_MSC_VER >= 1400)
	#define _CRT_SECURE_NO_WARNINGS
#endif

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>

#include "vd2\plugin\vdinputdriver.h"
#include "Unknown.h"

#include "InputOptionsFLV.h"
#include "Resource.h"

// These are in InputFileFLV.cpp
extern HMODULE g_hModule;
extern "C" const VDXPluginInfo *const * VDXAPIENTRY VDGetPluginInfo();

static const char FlvRegKey[] = "Software\\fccHandler\\Plugins\\FLV";


InputOptionsFLV::InputOptionsFLV()
{
	HKEY hKey;

	opts.len = sizeof(opts);

	// Defaults
	opts.fDeliverRawAAC = true;

	if (RegOpenKeyExA(HKEY_CURRENT_USER, FlvRegKey,
		0, KEY_READ, &hKey) == ERROR_SUCCESS)
	{
		DWORD lpcbData = opts.len;
		RegQueryValueExA(hKey, "Options", NULL, NULL, (LPBYTE)&opts, &lpcbData);
		RegCloseKey(hKey);
	}
}


InputOptionsFLV::~InputOptionsFLV()
{
}


void InputOptionsFLV::InitDialogTitle(HWND hDlg, char *pTitle)
{
	char buf[64];
	int i;

	uint32 ver = (*VDGetPluginInfo())->mVersion;

	i = _snprintf(buf, sizeof(buf), "FLV Plugin v%i.%i",
		(ver >> 24) & 0xFF, (ver >> 16) & 0xFF);

	if (i > 0 && i < sizeof(buf) && pTitle != NULL)
	{
		_snprintf(buf + i, sizeof(buf) - i, pTitle);
	}

	buf[sizeof(buf) - 1] = '\0';
	SetWindowTextA(hDlg, buf);
}


uint32 VDXAPIENTRY InputOptionsFLV::Write(void *buf, uint32 buflen)
{
	if (buf != NULL && buflen >= opts.len)
	{
		memcpy(buf, &opts, opts.len);
	}
	return opts.len;
}


INT_PTR CALLBACK InputOptionsFLV::OptionsDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	InputOptionsFLV *thisPtr;
	WORD wChosen;

	switch (message) {

	case WM_INITDIALOG:
		SetWindowLongPtrA(hDlg, DWLP_USER, lParam);
		thisPtr = (InputOptionsFLV *)lParam;
		InputOptionsFLV::InitDialogTitle(hDlg, " Options");
		CheckDlgButton(hDlg, IDC_ADD_ADTS_HEADERS, thisPtr->opts.fDeliverRawAAC? BST_UNCHECKED: BST_CHECKED);
		return TRUE;

	case WM_COMMAND:
		wChosen = LOWORD(wParam);
		if (wChosen == IDC_SAVE || wChosen == IDOK || wChosen == IDCANCEL)
		{
			thisPtr = (InputOptionsFLV *)GetWindowLongPtrA(hDlg, DWLP_USER);
			if (wChosen != IDCANCEL)
			{
				thisPtr->opts.fDeliverRawAAC = (IsDlgButtonChecked(hDlg, IDC_ADD_ADTS_HEADERS) == BST_UNCHECKED);
				if (wChosen == IDC_SAVE)
				{
					HKEY hKey;

					if (RegCreateKeyExA(HKEY_CURRENT_USER, FlvRegKey,
						0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS,
						NULL, &hKey, NULL) == ERROR_SUCCESS)
					{
						RegSetValueExA(hKey, "Options", 0, REG_BINARY,
							(CONST BYTE *)&thisPtr->opts, thisPtr->opts.len);

						RegCloseKey(hKey);
					}
				}
			}
			EndDialog(hDlg, TRUE);
			return TRUE;
		}
		break;

	default:
		break;
	}

	return FALSE;
}

void InputOptionsFLV::OptionsDlg(HWND hwndParent)
{
	DialogBoxParamA(g_hModule, MAKEINTRESOURCEA(IDD_OPTIONS),
		hwndParent, OptionsDlgProc, (LPARAM)this);
}

