#ifndef _VIDEOSOURCEFLV_H_
#define _VIDEOSOURCEFLV_H_


// Key frame bit (stored in size field)
#define FLV_KEYFRAME    0x80000000


class VideoTrackFLV
{
private:
	DataVector*         dv;

	int                 TransformAVCConfRec(const unsigned char *src, unsigned char *dst);

#ifdef FLV_VFR
	void                DeriveFrameRate();
#endif

public:
	VideoTrackFLV();
	~VideoTrackFLV();

	FLVSampleInfo*      sample_list;
	unsigned long*      sample_map;
	DWORD               dwActualLength;
	DWORD               dwPerceivedLength;

	BITMAPINFOHEADER    bmih;
	unsigned char       bFormat;
	HIC                 hCodec;

	DWORD               dwRate;
	DWORD               dwScale;

	unsigned char*      avc_dcr;
	int                 dcr_size;
	int                 avc_len;
	int                 avc_w;
	int                 avc_h;

	bool                ParseTag(WinFile &mFile, const FLVTag &tag);
	bool                IsDroppedFrame(unsigned long frame) const;
	bool                IsKey(unsigned long frame) const;
	bool                Finish();
};


class VideoSourceFLV : public vdxunknown<IVDXStreamSource>, public IVDXVideoSource {

public:
	VideoSourceFLV(InputFileFLV *const pp, VideoTrackFLV *const vt);
	~VideoSourceFLV();

	// IVDXUnknown
	int         VDXAPIENTRY AddRef();
	int         VDXAPIENTRY Release();
	void*       VDXAPIENTRY AsInterface(uint32 iid);

	// IVDXStreamSource
	void        VDXAPIENTRY GetStreamSourceInfo(VDXStreamSourceInfo&);
	bool        VDXAPIENTRY Read(sint64 lStart, uint32 lCount, void *lpBuffer, uint32 cbBuffer, uint32 *lBytesRead, uint32 *lSamplesRead);
	const void* VDXAPIENTRY GetDirectFormat();
	int         VDXAPIENTRY GetDirectFormatLen();
	ErrorMode   VDXAPIENTRY GetDecodeErrorMode();
	void        VDXAPIENTRY SetDecodeErrorMode(ErrorMode mode);
	bool        VDXAPIENTRY IsDecodeErrorModeSupported(ErrorMode mode);
	bool        VDXAPIENTRY IsVBR();
	sint64      VDXAPIENTRY TimeToPositionVBR(sint64 us);
	sint64      VDXAPIENTRY PositionToTimeVBR(sint64 samples);

	// IVDXVideoSource
	void        VDXAPIENTRY GetVideoSourceInfo(VDXVideoSourceInfo& info);
	bool        VDXAPIENTRY CreateVideoDecoderModel(IVDXVideoDecoderModel **);
	bool        VDXAPIENTRY CreateVideoDecoder(IVDXVideoDecoder **);
	void        VDXAPIENTRY GetSampleInfo(sint64 sample_num, VDXVideoFrameInfo& frameInfo);
	bool        VDXAPIENTRY IsKey(sint64 sample_num);
	sint64      VDXAPIENTRY GetFrameNumberForSample(sint64 sample_num);
	sint64      VDXAPIENTRY GetSampleNumberForFrame(sint64 frame_num);
	sint64      VDXAPIENTRY GetRealFrame(sint64 frame_num);
	sint64      VDXAPIENTRY GetSampleBytePosition(sint64 sample_num);

private:
	InputFileFLV*   const parentPtr;
	VideoTrackFLV*  const video;

	IVDXStreamSource::ErrorMode errmode;

	DWORD           dwGuess;
	DWORD           dwPrevSample;
	unsigned char*  frame_buffer;

	int UnwrapAVCFrame(unsigned char *dst, const unsigned char *src, int len);
};

#endif	// _VIDEOSOURCEFLV_H_
